#include "../../inc/camera.h"


Camera::Camera(){

    cfg = new rs2::config();
    pipe = new rs2::pipeline();
    align_to_depth = new rs2::align(RS2_STREAM_DEPTH);
    spatFilter = new rs2::spatial_filter();
    tempFilter = new rs2::temporal_filter();
    color_frame = new rs2::video_frame(rs2::frame());
    depth_frame = new rs2::depth_frame(rs2::frame());
}


Camera::~Camera(){
    delete align_to_depth;
    delete tempFilter;
    delete spatFilter;
    delete pipe;
    delete cfg;
    delete depth_frame;
    delete color_frame;
}


/*
*   Public
*/


int Camera::start(){
    
    // check for a device availible
    if(!cfg->can_resolve(*pipe)){
        //std::cout << "Error: No device found" << std::endl;
        return 1;
    }

    // Logs
    std::remove("./camera.logs");
    rs2::log_to_file(RS2_LOG_SEVERITY_DEBUG, "./camera.logs");

    // Settings
    // default camera resolution : 640x480
    cfg->enable_stream(RS2_STREAM_COLOR);
    cfg->enable_stream(RS2_STREAM_DEPTH);
    /*
    cfg->enable_stream(RS2_STREAM_COLOR, 1280, 720, RS2_FORMAT_RGB8, 30);
    cfg->enable_stream(RS2_STREAM_DEPTH, 1280, 720, RS2_FORMAT_Y16, 30);
    */
   
    rs2::depth_sensor sensor = pipe->start(*cfg).get_device().first<rs2::depth_sensor>();

    // Doc presets : https://dev.intelrealsense.com/docs/d400-series-visual-presets
    sensor.set_option(RS2_OPTION_VISUAL_PRESET, RS2_RS400_VISUAL_PRESET_HIGH_DENSITY);
    //sensor.set_option(RS2_OPTION_VISUAL_PRESET, RS2_RS400_VISUAL_PRESET_DEFAULT);
    

    // 5 = range mapped to unlimited
    spatFilter->set_option(RS2_OPTION_HOLES_FILL, 5);

    warmUpDepthLens();

    return 0;
}


void Camera::stop(){
    pipe->stop();
}


void Camera::capture(){

    auto frameset = pipe->wait_for_frames();

    // Trying to get frames from the depth perspective (aligned on depth camera)    
    frameset = align_to_depth->process(frameset);
    rs2::video_frame colorFrame = frameset.get_color_frame();
    rs2::depth_frame depthFrame = frameset.get_depth_frame();

    if(depthFrame && colorFrame){

        filterDepthFrame(depthFrame);

        // Values from depth matrix to real world
        // depth_frame.get_distance(x,y) == ((float)cv::Mat.at<uint16_t>(y, x) * depth_frame.get_units())
        color_frame->swap(colorFrame);
        depth_frame->swap(depthFrame);
    }
}

// get the coordinates of the pixel matching the point relative to the real world
cv::Point2i Camera::projectPointToPixel(cv::Point3f point3D){

    float point[3] = {point3D.x, point3D.y, point3D.z};
    float pixel[2];
    rs2_intrinsics intr = depth_frame->get_profile().as<rs2::video_stream_profile>().get_intrinsics();
    rs2_project_point_to_pixel(pixel, &intr, point);
    return cv::Point2i(pixel[0], pixel[1]);
}

// get  the point relative to the real world matching the coordinates of the pixel
cv::Point3f Camera::deprojectPixelToPoint(float coord[], float z){

    float p[3];
    rs2_intrinsics intr = depth_frame->get_profile().as<rs2::video_stream_profile>().get_intrinsics();
    rs2_deproject_pixel_to_point(p, &intr, coord, z);
    return cv::Point3f(p[0], p[1], p[2]);
}


void Camera::printCroppingMask(){
    cv::Rect mask = getCroppingMask();
    std::cout << "(" << mask.x << "," << mask.y << ") + " << mask.width << "x" << mask.height << std::endl;
}




/*
*   Private
*/

// Capture 30 frames to give autoexposure, etc. a chance to settle
void Camera::warmUpDepthLens()
{
    for (int i = 0; i < 30; ++i)
    {
        rs2::frameset data = pipe->wait_for_frames();
        auto frameDepth = data.get_depth_frame();
        filterDepthFrame(frameDepth);
    }
}

void Camera::filterDepthFrame(rs2::depth_frame &frameDepth)
{
    frameDepth = spatFilter->process(frameDepth);
    frameDepth = tempFilter->process(frameDepth);
}
